package com.persagy.apm.alarmservice.project.monitorindicatorconfig.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.apm.alarmservice.calculatemethod.model.CalculateMethod;
import com.persagy.apm.alarmservice.calculatemethod.model.dto.QueryCalculateMethodDTO;
import com.persagy.apm.alarmservice.calculatemethod.service.ICalculateMethodService;
import com.persagy.apm.alarmservice.group.management.service.GroupManagementService;
import com.persagy.apm.alarmservice.monitorindicator.model.MonitorIndicator;
import com.persagy.apm.alarmservice.monitorindicator.model.dto.QueryMonitorIndicatorDTO;
import com.persagy.apm.alarmservice.monitorindicator.service.IMonitorIndicatorService;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.dao.ProjectMonitorIndicatorConfigMapper;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.ConvertProjectMonitorIndicatorConfigTool;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.ProjectMonitorIndicatorConfig;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.dto.AddProjectMonitorIndicatorConfigDTO;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.dto.PageQueryProjectMonitorIndicatorConfigDTO;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.dto.QueryProjectMonitorIndicatorConfigDTO;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.model.dto.UpdateProjectMonitorIndicatorConfigDTO;
import com.persagy.apm.alarmservice.project.monitorindicatorconfig.service.IProjectMonitorIndicatorConfigService;
import com.persagy.apm.common.constant.enums.BoolEnum;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.model.dto.GroupManagementDTO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 项目监测指标配置(ProjectMonitorIndicatorConfig) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:46:56
 */
@Service
public class ProjectMonitorIndicatorConfigServiceImpl extends ServiceImpl<ProjectMonitorIndicatorConfigMapper, ProjectMonitorIndicatorConfig>
        implements IProjectMonitorIndicatorConfigService {

    @Autowired
    IMonitorIndicatorService monitorIndicatorService;
    @Autowired
    ICalculateMethodService calculateMethodService;
    @Autowired
    GroupManagementService groupManagementService;

    /**
     * 创建项目监测指标配置
     *
     * @return 项目监测指标配置主键
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    @Override
    public String createProjectMonitorIndicatorConfig(AddProjectMonitorIndicatorConfigDTO addProjectMonitorIndicatorConfigDTO) {
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = ConvertProjectMonitorIndicatorConfigTool.INSTANCE.convertAddDto2Entity(addProjectMonitorIndicatorConfigDTO);
        // 设置默认值
        setDefaultValue(projectMonitorIndicatorConfig);
        save(projectMonitorIndicatorConfig);
        return projectMonitorIndicatorConfig.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param projectMonitorIndicatorConfig 项目监测指标配置实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig) {
        projectMonitorIndicatorConfig.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 项目监测指标配置详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    @Override
    public ProjectMonitorIndicatorConfig queryProjectMonitorIndicatorConfigDetail(String id) {
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = getById(id);
        if (projectMonitorIndicatorConfig == null) {
            throw new IllegalArgumentException("查看ProjectMonitorIndicatorConfig详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return projectMonitorIndicatorConfig;
    }

    /**
     * 更新项目监测指标配置
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    @Override
    public String updateProjectMonitorIndicatorConfig(UpdateProjectMonitorIndicatorConfigDTO updateProjectMonitorIndicatorConfigDTO) {
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig =
                getConfigByObjAndIndicator(
                        updateProjectMonitorIndicatorConfigDTO.getObjId(),
                        updateProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId());
        projectMonitorIndicatorConfig.setCaculateMethodId(updateProjectMonitorIndicatorConfigDTO.getCalculateMethodId());
        projectMonitorIndicatorConfig.setModifier(AppContext.getContext().getAccountId());
        updateById(projectMonitorIndicatorConfig);
        return projectMonitorIndicatorConfig.getId();
    }

    /**
     * 校验项目监测指标配置是否可删除
     *
     * @param id 项目监测指标配置主键
     * @return 项目监测指标配置do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    public ProjectMonitorIndicatorConfig checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ProjectMonitorIndicatorConfig时发生异常，主键为空");
        }

        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = getById(id);

        if (projectMonitorIndicatorConfig == null) {
            throw new IllegalArgumentException("删除ProjectMonitorIndicatorConfig时发生异常，找不到要删除的数据，id:" + id);
        }

        return projectMonitorIndicatorConfig;
    }

    /**
     * 删除项目监测指标配置
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    @Override
    public void deleteProjectMonitorIndicatorConfig(String id) {
        // 校验是否可删除
        ProjectMonitorIndicatorConfig projectMonitorIndicatorConfig = checkDeletable(id);

        projectMonitorIndicatorConfig.setValid(ValidEnum.FALSE.getType());
        updateById(projectMonitorIndicatorConfig);
    }

    /**
     * 查询项目监测指标配置
     *
     * @return List<ProjectMonitorIndicatorConfig>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    @Override
    public List<ProjectMonitorIndicatorConfig> queryProjectMonitorIndicatorConfigList(
            QueryProjectMonitorIndicatorConfigDTO queryProjectMonitorIndicatorConfigDTO) {
        QueryWrapper<ProjectMonitorIndicatorConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectMonitorIndicatorConfig.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectMonitorIndicatorConfig.PROP_CREATIONTIME);

        if (queryProjectMonitorIndicatorConfigDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getObjId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_OBJ_ID, queryProjectMonitorIndicatorConfigDTO.getObjId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_MONITOR_INDICATOR_ID, queryProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getCaculateMethodId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_CACULATE_METHOD_ID, queryProjectMonitorIndicatorConfigDTO.getCaculateMethodId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectMonitorIndicatorConfigDTO.getProjectId())) {
                queryWrapper.like(ProjectMonitorIndicatorConfig.PROP_PROJECT_ID, queryProjectMonitorIndicatorConfigDTO.getProjectId());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询项目监测指标配置
     *
     * @return IPage<ProjectMonitorIndicatorConfig>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:56
     */
    @Override
    public IPage<ProjectMonitorIndicatorConfig> pageQueryProjectMonitorIndicatorConfig(
            PageQueryProjectMonitorIndicatorConfigDTO pageQueryProjectMonitorIndicatorConfigDTO) {
        // TODO: 2021/9/17
        // 获取分组下的监测指标，查询条件：设备类+监测指标id列表
        List<MonitorIndicator> monitorIndicators =  queryGroupIndicators(
                pageQueryProjectMonitorIndicatorConfigDTO.getClassCode(),
                pageQueryProjectMonitorIndicatorConfigDTO.getMonitorIndicatorId());
        // 查询分组下的设备, 查询条件：分组id+项目id+设备名称（模糊）
        String groupId = pageQueryProjectMonitorIndicatorConfigDTO.getGroupId();
        if (StringUtils.isBlank(groupId)) {
            throw new IllegalArgumentException("分组id不能为空");
        }
        String projectId = PoemsContext.getContext().getProjectId();
        GroupManagementDTO groupManagementDTO = new GroupManagementDTO();
        groupManagementService.equipsQuery(managementDTO);
        return null;
    }

    /**
     * 获取分组下的监测指标
     *
     * @param classCode 分组设备编码
     * @param indicatorIds 监测指标id列表
     * @return 分组下的监测指标
     * @author lixing
     * @version V1.0 2021/9/17 7:49 下午
     */
    private List<MonitorIndicator> queryGroupIndicators(String classCode, List<String> indicatorIds) {
        QueryMonitorIndicatorDTO queryMonitorIndicatorDTO = new QueryMonitorIndicatorDTO();
        queryMonitorIndicatorDTO.setClassCode(classCode);
        queryMonitorIndicatorDTO.setIds(indicatorIds);
        return monitorIndicatorService.queryMonitorIndicatorList(queryMonitorIndicatorDTO);
    }

    @Override
    public ProjectMonitorIndicatorConfig getConfigByObjAndIndicator(String objId, String monitorIndicatorId) {
        String projectId = PoemsContext.getContext().getProjectId();
        if (StringUtils.isBlank(objId) || StringUtils.isBlank(monitorIndicatorId)
                || StringUtils.isBlank(projectId)) {
            throw new IllegalArgumentException("" +
                    "ProjectMonitorIndicatorConfigServiceImpl.getConfigByObjAndIndicator：监测对象id、监测指标id、项目id不能为空");
        }


        // 根据监测对象id和监测指标id，获取设备监测指标的配置信息
        QueryProjectMonitorIndicatorConfigDTO queryProjectMonitorIndicatorConfigDTO =
                new QueryProjectMonitorIndicatorConfigDTO();
        queryProjectMonitorIndicatorConfigDTO.setObjId(objId);
        queryProjectMonitorIndicatorConfigDTO.setMonitorIndicatorId(monitorIndicatorId);

        queryProjectMonitorIndicatorConfigDTO.setProjectId(projectId);
        List<ProjectMonitorIndicatorConfig> projectMonitorIndicatorConfigs =
                queryProjectMonitorIndicatorConfigList(queryProjectMonitorIndicatorConfigDTO);

        if (!CollectionUtils.isEmpty(projectMonitorIndicatorConfigs)) {
            return projectMonitorIndicatorConfigs.get(0);
        }

        // 如果配置表中未查询到配置信息，获取监测指标的默认计算方式并返回
        return getDefaultConfig(objId, monitorIndicatorId, projectId);
    }

    /**
     * 获取设备监测指标的默认取值
     *
     * @param objId 监测对象id
     * @param monitorIndicatorId 监测指标id
     * @param projectId 项目id
     * @return 设备监测指标的默认取值
     * @author lixing
     * @version V1.0 2021/9/17 2:57 下午
     */
    private ProjectMonitorIndicatorConfig getDefaultConfig(String objId, String monitorIndicatorId, String projectId) {
        if (StringUtils.isBlank(objId) || StringUtils.isBlank(monitorIndicatorId) ||
                StringUtils.isBlank(projectId)) {
            throw new IllegalArgumentException("" +
                    "ProjectMonitorIndicatorConfigServiceImpl.getDefaultConfig：监测对象id、监测指标id、项目id不能为空");
        }

        // 获取监测指标的默认计算方式
        QueryCalculateMethodDTO queryCalculateMethodDTO = new QueryCalculateMethodDTO();
        queryCalculateMethodDTO.setMonitorIndicatorId(monitorIndicatorId);
        queryCalculateMethodDTO.setIsDefault(BoolEnum.TRUE.getType());
        List<CalculateMethod> calculateMethods = calculateMethodService.queryCalculateMethodList(queryCalculateMethodDTO);
        String calculateMethodId = null;
        if (!CollectionUtils.isEmpty(calculateMethods)) {
            calculateMethodId = calculateMethods.get(0).getId();
        }

        return new ProjectMonitorIndicatorConfig(
                objId, monitorIndicatorId, calculateMethodId, projectId);
    }
}
